/*
    Copyright 2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef WIN32
    #define stricmp strcasecmp
    #define stristr strcasestr
#endif

#ifndef u_char
    typedef unsigned char   u_char;
#endif


#define VER     "0.1.1"



void delimit(u_char *data);
void gftp_descramble(u_char *data);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd;
    u_char  buff[4096],
            *fname,
            *p;


    setbuf(stdout, NULL);

    fputs("\n"
        "Gftp bookmarks passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <bookmarks>\n"
            "\n"
            "Example: ~/.gftp/bookmarks\n"
            "\n", argv[0]);
        fputs("\n  press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    fname = argv[1];
    fd = fopen(fname, "rb");
    if(!fd) std_err();

    while(fgets(buff, sizeof(buff), fd)) {
        delimit(buff);
	if(buff[0] == '[') {
	    printf("\n");
	    continue;
	}
	p = strchr(buff, '=');
	if(!p) continue;
	*p++ = 0;
	if(!stricmp(buff, "hostname")) {
	    printf("  hostname: %s\n", p);
	} else if(!stricmp(buff, "username")) { // thanx Roberto Berto!!!
	    printf("  username: %s\n", p);
	} if(!stricmp(buff, "password")) {
	    gftp_descramble(p);
	    printf("  password: %s\n", p);
	}
    }

    fclose(fd);
    fputs("\n  press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void delimit(u_char *data) {
    for(; *data && (*data != '\r') && (*data != '\n'); data++);
    *data = 0;
}



void gftp_descramble(u_char *data) {
    u_char  *in,
            *out;

    if(*data != '$') return;

    for(in = data + 1, out = data; in[0] && in[1]; in += 2, out++) {
    	if((in[0] & 0xc3) != 0x41) break;
    	if((in[1] & 0xc3) != 0x41) break;
        *out = ((in[0] & 0x3c) << 2) | ((in[1] & 0x3c) >> 2);
    }
    *out = 0;
}



void std_err(void) {
    perror("\nError");
    fputs("\n  press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}


